@@ -17,8 +17,8 @@ from jx_base.expressions.literal import is_literal
 class GetOp(Expression):
     has_simple_form = True
 
-    def __init__(self, term):
-        Expression.__init__(self, term)
+    def __init__(self, *term):
+        Expression.__init__(self, *term)
         self.var = term[0]
         self.offsets = term[1:]
 
@@ -35,6 +35,4 @@ class GetOp(Expression):
         return output
 
     def map(self, map_):
-        return self.lang[
-            GetOp([self.var.map(map_)] + [o.map(map_) for o in self.offsets])
-        ]
+        return GetOp(self.var.map(map_), *(o.map(map_) for o in self.offsets))
