@@ -21,8 +21,8 @@ from mo_logs import Log
 class RowsOp(Expression):
     has_simple_form = True
 
-    def __init__(self, term):
-        Expression.__init__(self, term)
+    def __init__(self, *term):
+        Expression.__init__(self, *term)
         self.var, self.offset = term
         if is_op(self.var, Variable):
             if is_op(self.var, Variable) and not any(
@@ -44,4 +44,4 @@ class RowsOp(Expression):
         return self.var.vars() | self.offset.vars() | {"rows", "rownum"}
 
     def map(self, map_):
-        return self.lang[RowsOp([self.var.map(map_), self.offset.map(map_)])]
+        return RowsOp(self.var.map(map_), self.offset.map(map_))
