@@ -12,27 +12,25 @@ from __future__ import absolute_import, division, unicode_literals
 
 from jx_base.expressions.expression import Expression
 from jx_base.expressions.false_op import FALSE
-from mo_json import INTEGER
+from mo_json import T_INTEGER
 
 
 class SqlSubstrOp(Expression):
-    data_type = INTEGER
+    _data_type = T_INTEGER
 
-    def __init__(self, params):
+    def __init__(self, *params):
         Expression.__init__(self, params)
         self.value, self.start, self.length = params
 
     def __data__(self):
-        return {
-            "sql.substr": [
-                self.value.__data__(),
-                self.start.__data__(),
-                self.length.__data__(),
-            ]
-        }
+        return {"sql.substr": [
+            self.value.__data__(),
+            self.start.__data__(),
+            self.length.__data__(),
+        ]}
 
     def vars(self):
         return self.value.vars() | self.start.vars() | self.length.vars()
 
-    def missing(self):
+    def missing(self, lang):
         return FALSE
