@@ -10,9 +10,8 @@
 from __future__ import absolute_import, division, unicode_literals
 
 from jx_base.expressions import MaxOp as MaxOp_
-from jx_python.expressions._utils import Python
 
 
 class MaxOp(MaxOp_):
-    def to_python(self, not_null=False, boolean=False, many=False):
-        return "max([" + (",".join(Python[t].to_python() for t in self.terms)) + "])"
+    def to_python(self):
+        return "max([" + ",".join((t).to_python() for t in self.terms) + "])"
