@@ -11,15 +11,12 @@ from __future__ import absolute_import, division, unicode_literals
 
 from jx_base.expressions import OrOp as OrOp_
 from jx_python.expressions import _utils
-from jx_python.expressions._utils import Python
-from jx_python.expressions.boolean_op import BooleanOp
+from jx_python.expressions.to_boolean_op import ToBooleanOp
 
 
 class OrOp(OrOp_):
-    def to_python(self, not_null=False, boolean=False, many=False):
-        return " or ".join(
-            "(" + BooleanOp(Python[t]).to_python() + ")" for t in self.terms
-        )
+    def to_python(self):
+        return " or ".join("(" + ToBooleanOp(t).to_python() + ")" for t in self.terms)
 
 
 _utils.OrOp = OrOp
