@@ -10,14 +10,13 @@
 from __future__ import absolute_import, division, unicode_literals
 
 from jx_base.expressions import TupleOp as TupleOp_
-from jx_python.expressions._utils import Python
 
 
 class TupleOp(TupleOp_):
-    def to_python(self, not_null=False, boolean=False, many=False):
+    def to_python(self):
         if len(self.terms) == 0:
             return "tuple()"
         elif len(self.terms) == 1:
-            return "(" + Python[self.terms[0]].to_python() + ",)"
+            return "(" + (self.terms[0]).to_python() + ",)"
         else:
-            return "(" + (",".join(Python[t].to_python() for t in self.terms)) + ")"
+            return "(" + ",".join((t).to_python() for t in self.terms) + ")"
